@@ -1,6 +1,8 @@
 #include <rm.hpp>
 
 #include <kdb.hpp>
+#include <cmdline.hpp>
+#include <print.hpp>
 
 #include <iostream>
 
