@@ -13,8 +13,8 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef HAVE_KDBCONFIG_H
+#include "kdbconfig.h"
 #endif
 
 #if DEBUG && HAVE_STDIO_H
@@ -95,11 +95,9 @@ char *keyNameGetOneLevel(const char *name, size_t *size) {
 	/* now see where this basename ends handling escaped chars with '\' */
 	while (real[cursor] && ! end) {
 		switch (real[cursor]) {
-#ifndef WIN32
 			case '\\':
 				escapeNext=1;
 				break;
-#endif
 			case PATH_SEPARATOR:
 				if (! escapeNext) end=1;
 			default:
@@ -277,8 +275,9 @@ ssize_t keyNameGetRootNameSize(const char *keyName) {
 ssize_t keyGetFullRootNameSize(const Key *key) {
 	size_t size=0;
 
-	if (keyIsUser(key)) {
-		if (key->owner) size=kdbiStrLen(key->owner);
+	if (keyIsUser(key))
+	{
+		if (keyGetMeta(key, "owner")) size=keyGetValueSize(keyGetMeta(key, "owner"));
 		
 		return size+sizeof("user");
 	} else {
@@ -326,13 +325,11 @@ ssize_t keyGetFullRootName(const Key *key, char *returned, size_t maxSize) {
 	
 	rootSize = keyGetRootNameSize(key)-1;
 	strncpy(returned,key->key, rootSize); /* copy "user" or "system" */
-	if (keyIsUser(key)) {
+	if (keyIsUser(key) && keyGetMeta(key, "owner"))
+	{
 		cursor = returned + rootSize;
 		*cursor = ':'; cursor++;
-		if (key->owner)
-		{
-			strncpy (cursor, key->owner, size - rootSize - 1); /* -1 for : */
-		}
+		strncpy (cursor, keyValue(keyGetMeta(key, "owner")), size - rootSize - 1); /* -1 for : */
 	} else {
 		returned[rootSize]=0;
 	}
@@ -412,7 +409,7 @@ free (parentName);
  * @ingroup keyname
  */
 ssize_t keyGetParentName(const Key *key, char *returnedParent, size_t maxSize) {
-	ssize_t parentSize;
+	size_t parentSize;
 
 	parentSize=keyGetParentNameSize(key);
 
@@ -509,12 +506,6 @@ int keyInit(Key *key)
 {
 	memset(key,0,sizeof(Key));
 
-	key->type = KEY_TYPE_UNDEFINED;
-
-	key->uid = (uid_t)-1;
-	key->gid = (gid_t)-1;
-
-	key->mode = KEY_DEF_MODE;
 	return 0;
 }
 
