@@ -51,8 +51,8 @@
 
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef HAVE_KDBCONFIG_H
+#include "kdbconfig.h"
 #endif
 
 #if DEBUG && HAVE_STDIO_H
@@ -171,6 +171,11 @@ ssize_t keyGetNameSize(const Key *key)
  * value passed by accident. Of course malloc is not
  * as failure tolerant and will try to allocate.
  *
+ * @code
+char *getBack = malloc (keyGetNameSize(key));
+keyGetName(key, getBack, keyGetNameSize(key));
+ * @endcode
+ *
  * @return number of bytes written to @p returnedName
  * @return 1 when only a null was written
  * @return -1 when keyname is longer then maxSize or 0 or any NULL pointer
@@ -253,20 +258,20 @@ ssize_t keySetName(Key *key, const char *newName)
 	size_t size=0;
 
 	if (!key) return -1;
+	if (key->flags & KEY_FLAG_RO) return -1;
 
 	if (key->key) free (key->key);
 	key->key = 0;
 	key->keySize=1; /* equal to length plus room for \\0 */
 
 	/* handle null new key name, removing the old name */
-	if (!newName || !(length=kdbiStrLen(newName)-1))
+	if (!newName || !(length=elektraStrLen(newName)-1))
 	{
 		if (key->key) {
 			key->keySize = 0;
 			free(key->key);
 			key->key=0;
 		}
-		key->type = 0;
 		return 0;
 	}
 
@@ -292,12 +297,14 @@ ssize_t keySetName(Key *key, const char *newName)
 				/* handle "user:*" */
 				if (ownerLength > 0)
 				{
-					p=realloc(key->owner,ownerLength+1);
+					char *owner;
+					p=elektraMalloc(ownerLength+1);
 					if (NULL==p) goto error_mem;
-					key->owner=p;
-					strncpy(key->owner,newName+userLength+1,ownerLength);
-					key->owner[ownerLength]=0;
-					key->ownerSize = ownerLength+1;
+					owner=p;
+					strncpy(owner,newName+userLength+1,ownerLength);
+					owner[ownerLength]=0;
+					keySetOwner(key, owner);
+					elektraFree (owner);
 				}
 				key->keySize+=length-ownerLength-1;  /* -1 is for the ':' */
 			} else if (*(newName+userLength)!=PATH_SEPARATOR) {
@@ -315,23 +322,6 @@ ssize_t keySetName(Key *key, const char *newName)
 			key->keySize+=userLength;
 		}
 
-		if (!key->owner)
-		{
-			char *envVar;
-
-			envVar = getenv("KDB_USER");
-			if ( envVar ) {
-				keySetOwner(key, envVar);
-			} else {
-				envVar = getenv ("USER");
-				if ( envVar ) {
-					keySetOwner(key, envVar);
-				} else {
-					keySetOwner(key, NULL);
-				}
-			}
-		}
-
 		rootLength  = userLength;
 	} else if (keyNameIsSystem(newName)) {
 		/* handle "system*" */
@@ -416,7 +406,7 @@ ssize_t keySetName(Key *key, const char *newName)
 
 	key->flags |= KEY_FLAG_SYNC;
 
-	key->keySize ++; /*for \0 ending*/
+	key->keySize ++; /*for \\0 ending*/
 	return key->keySize;
 
 error_mem:
@@ -446,13 +436,13 @@ ssize_t keyGetFullNameSize(const Key *key)
 
 	if (!key->key) return 1;
 
-	returnedSize=kdbiStrLen(key->key);
+	returnedSize=elektraStrLen(key->key);
 
-	if (keyNameIsUser(key->key) && key->owner)
-		returnedSize+=kdbiStrLen(key->owner);
+	if (keyNameIsUser(key->key) && keyGetMeta(key, "owner"))
+		returnedSize+=keyGetOwnerSize(key);
 
 	/*
-	   After 2 kdbiStrLen() calls looks like we counted one more NULL.
+	   After 2 elektraStrLen() calls looks like we counted one more NULL.
 	   But we need this byte count because a full key name has an
 	   additional ':' char.
 	*/
@@ -480,6 +470,7 @@ ssize_t keyGetFullName(const Key *key, char *returnedName, size_t maxSize)
 	size_t userSize=sizeof("user")-1;
 	size_t ownerSize;
 	ssize_t length;
+	ssize_t maxSSize;
 	char *cursor;
 
 	if (!key) return -1;
@@ -487,6 +478,7 @@ ssize_t keyGetFullName(const Key *key, char *returnedName, size_t maxSize)
 	if (!maxSize) return -1;
 
 	if (maxSize > SSIZE_MAX) return -1;
+	maxSSize = maxSize;
 
 	length=keyGetFullNameSize(key);
 	if (length == 1) {
@@ -495,7 +487,7 @@ ssize_t keyGetFullName(const Key *key, char *returnedName, size_t maxSize)
 		return length;
 	}
 	else if (length < 0) return length;
-	else if (length > maxSize) {
+	else if (length > maxSSize) {
 		/* errno=KDB_ERR_TRUNC; */
 		return -1;
 	}
@@ -505,10 +497,11 @@ ssize_t keyGetFullName(const Key *key, char *returnedName, size_t maxSize)
 	{
 		strncpy(cursor,key->key,userSize);
 		cursor+=userSize;
-		if (key->owner) {
+		if (keyGetMeta(key, "owner"))
+		{
 			*cursor=':'; ++cursor;
-			ownerSize=kdbiStrLen(key->owner)-1;
-			strcpy(cursor,key->owner);
+			ownerSize=keyGetValueSize(keyGetMeta(key, "owner"))-1;
+			strncpy(cursor,keyValue(keyGetMeta(key, "owner")),ownerSize);
 			cursor+=ownerSize;
 		}
 		strcpy(cursor,key->key+userSize);
@@ -544,7 +537,7 @@ keyDel(key);
  *
  * @param key the object to obtain the basename from
  * @return a pointer to the basename
- * @return "" on null pointer or when the key has no name
+ * @return "" when the key has no (base)name
  * @return 0 on NULL pointer
  * @see keyGetBaseName(), keyGetBaseNameSize()
  * @see keyName() to get a pointer to the name
@@ -692,6 +685,8 @@ ssize_t keyGetBaseName(const Key *key, char *returned, size_t maxSize)
  * @warning You should not change a keys name once it belongs to a keyset.
  * See ksSort() for more information.
  *
+ * TODO: does not recognice .. and . in the string!
+ *
  * @param key the key object to work with
  * @param baseName the string to append to the name
  * @return the size in bytes of the new key name including the ending NULL
@@ -744,6 +739,8 @@ ssize_t keyAddBaseName(Key *key, const char *baseName)
  * @warning You should not change a keys name once it belongs to a keyset.
  * See ksSort() for more information.
  *
+ * TODO: does not work with .. and .
+ *
  * @param key the key object to work with
  * @param baseName the string used to overwrite the basename of the key
  * @return the size in bytes of the new key name
@@ -824,14 +821,18 @@ keyOwner(key); // you would expect "" here
  */
 const char *keyOwner(const Key *key)
 {
-	if (!key) return 0;
+	const char *owner;
 
-	if (!key->owner) {
+	if (!key) return 0;
+	owner = keyValue(keyGetMeta(key, "owner"));
+
+	if (!owner)
+	{
 		/*errno=KDB_ERR_NOKEY;*/
 		return "";
 	}
 
-	return key->owner;
+	return owner;
 }
 
 
@@ -865,14 +866,18 @@ buffer = malloc (keyGetOwnerSize (key));
  */
 ssize_t keyGetOwnerSize(const Key *key)
 {
+	ssize_t size;
 	if (!key) return -1;
 
-	if (!key->owner) {
-		/*errno=KDB_ERR_NOOWNER;*/
+	size = keyGetValueSize(keyGetMeta (key, "owner"));
+
+	if (!size || size == -1)
+	{
+		/*errno=KDB_ERR_NODESC;*/
 		return 1;
 	}
 
-	return key->ownerSize;
+	return size;
 }
 
 
@@ -892,7 +897,7 @@ ssize_t keyGetOwnerSize(const Key *key)
  * UIDs are related to mode controls of a key.
  *
  * @param key the object to work with
- * @param returned a pre-allocated space to store the owner
+ * @param returnedOwner a pre-allocated space to store the owner
  * @param maxSize maximum number of bytes that fit returned
  * @return number of bytes written to buffer
  * @return 1 if there is no owner
@@ -901,26 +906,32 @@ ssize_t keyGetOwnerSize(const Key *key)
  * @see keySetName(), keySetOwner(), keyOwner(), keyGetFullName()
  * @ingroup keyname
  */
-ssize_t keyGetOwner(const Key *key, char *returned, size_t maxSize)
+ssize_t keyGetOwner(const Key *key, char *returnedOwner, size_t maxSize)
 {
+	const char *owner;
+	size_t ownerSize;
 	if (!key) return -1;
-	if (!maxSize) return -1;
-	if (!returned) return -1;
 
+	if (!maxSize) return -1;
+	if (!returnedOwner) return -1;
 	if (maxSize > SSIZE_MAX) return -1;
 
-	if (!key->owner) {
-		/*errno=KDB_ERR_NOOWNER;*/
-		returned[0]=0;
+	owner = keyValue(keyGetMeta(key, "owner"));
+	ownerSize = keyGetValueSize(keyGetMeta(key, "owner"));
+
+	if (!owner)
+	{
+		/*errno=KDB_ERR_NODESC;*/
+		returnedOwner[0]=0;
 		return 1;
 	}
 
-	if (maxSize < key->ownerSize)
-	{
+	strncpy(returnedOwner,owner,maxSize);
+	if (maxSize < ownerSize) {
 		/*errno=KDB_ERR_TRUNC;*/
 		return -1;
-	} else strcpy(returned,key->owner);
-	return key->ownerSize;
+	}
+	return ownerSize;
 }
 
 
@@ -936,44 +947,22 @@ ssize_t keyGetOwner(const Key *key, char *returned, size_t maxSize)
  * the call.
  *
  * @param key the key object to work with
- * @param owner the owner (or user name)
+ * @param newOwner the string which describes the owner of the key
  * @return the number of bytes actually saved including final NULL
  * @return 1 when owner is freed (by setting 0 or "")
  * @return -1 on null pointer or memory problems
  * @see keySetName(), keyGetOwner(), keyGetFullName()
  * @ingroup keyname
  */
-ssize_t keySetOwner(Key *key, const char *owner)
+ssize_t keySetOwner(Key *key, const char *newOwner)
 {
 	if (!key) return -1;
-
-	if (owner && (key->ownerSize=kdbiStrLen(owner)) > 1)
+	if (!newOwner || *newOwner==0)
 	{
-		if (key->owner) {
-			char *p=0;
-			p=realloc(key->owner,key->ownerSize);
-			if (NULL==p) {
-				/*errno=KDB_ERR_NOMEM;*/
-				return -1;
-			}
-			key->owner=p;
-		} else {
-			key->owner=malloc(key->ownerSize);
-			if (!key->owner) {
-				/*errno=KDB_ERR_NOMEM;*/
-				return -1;
-			}
-		}
-
-		strcpy(key->owner,owner);
-		key->flags |= KEY_FLAG_SYNC;
-	} else if (key->owner) {
-		free(key->owner);
-		key->ownerSize=1;
-		key->owner=0;
-		key->flags |= KEY_FLAG_SYNC;
+		keySetMeta (key, "owner", 0);
+		return 1;
 	}
-	return key->ownerSize;
+
+	keySetMeta (key, "owner", newOwner);
+	return keyGetOwnerSize (key);
 }
-
-
