@@ -18,7 +18,8 @@
  * @defgroup keyset KeySet :: Class Methods
  * @brief Methods to manipulate KeySets.
  *
- * A KeySet is a unsorted set of keys.
+ * A KeySet is a sorted set of keys.
+ * So the correct name actually would be KeyMap.
  *
  * Terminate with ksNew(0) or ksNew(20, ..., KS_END)
  * This is because there is a list of Key* required and
@@ -61,7 +62,8 @@ ksAppendKey(myConfig, keyNew("user/name4", 0));
 
 Key *current;
 ksRewind(myConfig);
-while ((current=ksNext(myConfig))!=0) {
+while ((current=ksNext(myConfig))!=0)
+{
 	printf("Key name is %s.\n", keyName (current));
 }
 ksDel (myConfig); // delete keyset and all keys appended
@@ -159,7 +161,8 @@ ksDel (config);
  * @return a ready to use KeySet object
  * @return 0 on memory error
  */
-KeySet *ksNew(size_t alloc, ...) {
+KeySet *ksNew(size_t alloc, ...)
+{
 	KeySet *ks;
 	va_list va;
 
@@ -317,7 +320,9 @@ int ksDel(KeySet *ks)
 	return rc;
 }
 
-/*
+/**
+ * @internal
+ *
  * KeySet object cleaner.
  *
  * Will keyDel() all contained keys, reset internal pointers and counters.
@@ -359,20 +364,13 @@ int ksClear(KeySet *ks)
 
 
 
-/*
- * Returns 0.
- *
- * @deprecated dont use
+/**
+ * @internal
  *
+ * Used as a callback by the qsort() function
  */
-int ksNeedSort (const KeySet *ks)
+static int keyCmpInternal(const void *p1, const void *p2)
 {
-	return 0;
-}
-
-
-/* Used as a callback by the qsort() function */
-static int keyCmpInternal(const void *p1, const void *p2) {
 	Key *key1=*(Key **)p1;
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
@@ -456,7 +454,7 @@ Key *k2 = keyNew("user/a", KEY_OWNER, "max", KEY_END);
 // keyCmp(k2,k1) > 0
  * @endcode
  *
- * @warning dont try to strcmp the keyName() yourself because
+ * @warning Do not try to strcmp the keyName() yourself because
  *      the used strcmp implementation is allowed to differ from
  *      simple ascii comparison.
  *
@@ -480,7 +478,10 @@ int keyCmp (const Key *k1, const Key *k2)
 	return keyCmpInternal(&k1, &k2);
 }
 
-/**Checks if KeySet needs sync.
+/**
+ * @internal
+ *
+ * Checks if KeySet needs sync.
  *
  * When keys are changed this is reflected into keyNeedSync().
  *
@@ -529,6 +530,8 @@ ssize_t ksGetSize(const KeySet *ks)
 
 
 /**
+ * @internal
+ *
  * Binary search in a keyset.
  *
  * @code
@@ -623,7 +626,7 @@ ssize_t ksSearchInternal(const KeySet *ks, const Key *toAppend)
  * @return -1 if insertion failed, the key will be deleted then.
  * @param ks KeySet that will receive the key
  * @param toAppend Key that will be appended to ks
- * @see ksInsert(), ksInsertKeys(), ksAppend(), keyNew(), ksDel()
+ * @see ksAppend(), keyNew(), ksDel()
  * @see keyIncRef()
  *
  */
@@ -704,7 +707,7 @@ ssize_t ksAppendKey(KeySet *ks, Key *toAppend)
  * @return -1 on NULL pointers
  * @param ks the KeySet that will receive the keys
  * @param toAppend the KeySet that provides the keys that will be transfered
- * @see ksAppendKey(), ksInsert(), ksInsertKeys()
+ * @see ksAppendKey()
  * 
  */
 ssize_t ksAppend(KeySet *ks, const KeySet *toAppend)
@@ -732,7 +735,9 @@ ssize_t ksAppend(KeySet *ks, const KeySet *toAppend)
 static int keyCompareByNameOwner(const void *p1, const void *p2);
 
 
-/*
+/**
+ * @internal
+ *
  * Returns the previous Key in a KeySet.
  *
  * KeySets have an internal cursor that can be reset with ksRewind(). Every
@@ -762,6 +767,8 @@ static Key *ksPrev(KeySet *ks)
 
 
 /**
+ * @internal
+ *
  * Copies all Keys until the end of the array from a position
  * in the array to an position in the array.
  *
@@ -1033,7 +1040,6 @@ Key *ksNext(KeySet *ks)
  * @return pointer to the Key pointed by @p ks's cursor
  * @return 0 on NULL pointer
  * @see ksNext(), ksRewind()
- * @see kdbMonitorKeys() for a usage example
  */
 Key *ksCurrent(const KeySet *ks)
 {
@@ -1226,7 +1232,8 @@ int ksSetCursor(KeySet *ks, cursor_t cursor)
  *    Looking up Keys inside KeySets       *
  *******************************************/
 
-static int keyCompareByName(const void *p1, const void *p2) {
+static int keyCompareByName(const void *p1, const void *p2)
+{
 	Key *key1=*(Key **)p1;
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
@@ -1235,7 +1242,8 @@ static int keyCompareByName(const void *p1, const void *p2) {
 	return strcmp(name1, name2);
 }
 
-static int keyCompareByNameCase(const void *p1, const void *p2) {
+static int keyCompareByNameCase(const void *p1, const void *p2)
+{
 	Key *key1=*(Key **)p1;
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
@@ -1244,7 +1252,8 @@ static int keyCompareByNameCase(const void *p1, const void *p2) {
 	return elektraStrCaseCmp(name1, name2);
 }
 
-static int keyCompareByNameOwner(const void *p1, const void *p2) {
+static int keyCompareByNameOwner(const void *p1, const void *p2)
+{
 	Key *key1=*(Key **)p1;
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
@@ -1261,7 +1270,8 @@ static int keyCompareByNameOwner(const void *p1, const void *p2) {
 }
 
 
-static int keyCompareByNameOwnerCase(const void *p1, const void *p2) {
+static int keyCompareByNameOwnerCase(const void *p1, const void *p2)
+{
 	Key *key1=*(Key **)p1;
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
@@ -1303,14 +1313,14 @@ static int keyCompareByNameOwnerCase(const void *p1, const void *p2) {
  * Cascading is done if the first character is a /. This leads to ignoring
  * the prefix like system/ and user/.
  * @code
-        if (kdbGet(handle, "user/myapp", myConfig, 0 ) == -1)
-                ErrorHandler ("Could not get Keys");
+if (kdbGet(handle, "user/myapp", myConfig, 0 ) == -1)
+	errorHandler ("Could not get Keys");
 
-        if (kdbGet(handle, "system/myapp", myConfig, 0 ) == -1)
-                ErrorHandler ("Could not get Keys");
+if (kdbGet(handle, "system/myapp", myConfig, 0 ) == -1)
+	errorHandler ("Could not get Keys");
 
-        if ((myKey = ksLookup(myConfig, key, 0)) == NULL)
-                ErrorHandler ("Could not Lookup Key");
+if ((myKey = ksLookup(myConfig, key, 0)) == NULL)
+	errorHandler ("Could not Lookup Key");
  * @endcode
  *
  * This is the way multi user Programs should get there configuration and
@@ -1397,7 +1407,8 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
 
 	if (options & KDB_O_NOALL)
 	{
-		while ((current=ksNext(ks)) != 0) {
+		while ((current=ksNext(ks)) != 0)
+		{
 			if ((options & KDB_O_WITHOWNER) && (options & KDB_O_NOCASE))
 			{
 				if (!keyCompareByNameOwnerCase(&key, &current)) break;
@@ -1487,17 +1498,32 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
  * Cascading is done if the first character is a /. This leads to ignoring
  * the prefix like system/ and user/.
  * @code
-        if (kdbGetByName(handle, "/sw/myapp/current", myConfig, 0 ) == -1)
-                ErrorHandler ("Could not get Keys");
+if (kdbGet(handle, "user/sw/myapp/current", myConfig, parentKey ) == -1)
+	errorHandler ("Could not get Keys", parentKey);
 
-        if ((myKey = ksLookupByName (myConfig, "/myapp/current/key", 0)) == NULL)
-                ErrorHandler ("Could not Lookup Key");
+if (kdbGet(handle, "system/sw/myapp/current", myConfig, parentKey ) == -1)
+	errorHandler ("Could not get Keys", parentKey);
+
+if ((myKey = ksLookupByName (myConfig, "/myapp/current/key", 0)) == NULL)
+	errorHandler ("Could not Lookup Key");
  * @endcode
- * 
+ *
  * This is the way multi user Programs should get there configuration and
  * search after the values. It is guaranteed that more namespaces can be
  * added easily and that all values can be set by admin and user.
  *
+ * It is up to the application to implement a sophisticated cascading
+ * algorithm, for e.g. a list of profiles (specific, group and fallback):
+ * @code
+if ((myKey = ksLookupByName (myConfig, "/myapp/current/specific/key", 0)) == NULL)
+	if ((myKey = ksLookupByName (myConfig, "/myapp/current/group/key", 0)) == NULL)
+		if ((myKey = ksLookupByName (myConfig, "/myapp/current/fallback/key", 0)) == NULL)
+			errorHandler ("All fallbacks failed to lookup key");
+ * @endcode
+ *
+ * Note that for every profile both the user and the system key are
+ * searched. The first key found will be used.
+ *
  * @section fullsearch Full Search
  *
  * When KDB_O_NOALL is set the keyset will be only searched from ksCurrent()
@@ -1586,7 +1612,8 @@ Key *ksLookupByName(KeySet *ks, const char *name, option_t options)
  * @par Example:
  * @code
 ksRewind(ks);
-while (key=ksLookupByString(ks,"my value",0)) {
+while (key=ksLookupByString(ks,"my value",0))
+{
 	// show all keys which value="my value"
 	keyToStream(key,stdout,0);
 }
@@ -1712,7 +1739,9 @@ Key *ksLookupByBinary(KeySet *ks, const void *value, size_t size,
 
 
 
-/*
+/**
+ * @internal
+ *
  * Calculates the common parent to all keys in @p ks.
  *
  * This is a c-helper function, you need not implement it in bindings.
@@ -1739,8 +1768,6 @@ Key *ksLookupByBinary(KeySet *ks, const void *value, size_t size,
  *
  * No common parent is possible, so @p returnedCommonParent will contain nothing.
  *
- * This method will work correctly only sorted KeySets.
- *
  * @param working the Keyset to work with
  * @param returnedCommonParent a pre-allocated buffer that will receive the
  *        common parent, if found
@@ -1748,7 +1775,8 @@ Key *ksLookupByBinary(KeySet *ks, const void *value, size_t size,
  * @return size in bytes of the parent name, or 0 if there is no common parent,
  *         or -1 to indicate an error, then @p errno must be checked.
  */
-ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent, size_t maxSize) {
+ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent, size_t maxSize)
+{
 	size_t parentSize=0;
 	Key *current=0;
 	cursor_t init;
@@ -1765,7 +1793,8 @@ ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent,
 
 	ksRewind(ks);
 	current = ksNext(ks);
-	if (keyGetNameSize(current) > sMaxSize) {
+	if (keyGetNameSize(current) > sMaxSize)
+	{
 		/*errno=KDB_ERR_TRUNC;*/
 		returnedCommonParent[0]=0;
 		return -1;
@@ -1774,18 +1803,22 @@ ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent,
 	strcpy(returnedCommonParent,keyName(current));
 	parentSize=elektraStrLen(returnedCommonParent);
 
-	while (*returnedCommonParent) {
+	while (*returnedCommonParent)
+	{
 		ksRewind(ks);
-		while ((current = ksNext(ks)) != 0) {
+		while ((current = ksNext(ks)) != 0)
+		{
 			/* Test if a key doesn't match */
 			if (memcmp(returnedCommonParent,keyName(current),parentSize-1)) break;
 		}
-		if (current) {
+		if (current)
+		{
 			/* some key failed to be a child */
 			/* parent will be the parent of current parent... */
 			char *delim=0;
 
-			if ((delim=strrchr(returnedCommonParent,PATH_SEPARATOR))) {
+			if ((delim=strrchr(returnedCommonParent,PATH_SEPARATOR)))
+			{
 				*delim=0;
 				parentSize=elektraStrLen(returnedCommonParent);
 			} else {
@@ -1811,7 +1844,9 @@ ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent,
  *********************************************************************/
 
 
-/*
+/**
+ * @internal
+ *
  * Resize keyset.
  *
  * For internal useage only.
@@ -1880,7 +1915,9 @@ int ksResize (KeySet *ks, size_t alloc)
 	return 1;
 }
 
-/*
+/**
+ * @internal
+ *
  * Returns current allocation size.
  *
  * This is the maximum size before a reallocation
@@ -1895,7 +1932,9 @@ size_t ksGetAlloc (const KeySet *ks)
 
 
 
-/*
+/**
+ * @internal
+ *
  * KeySet object initializer.
  *
  * You should always use ksNew() instead of ksInit().
@@ -1903,12 +1942,12 @@ size_t ksGetAlloc (const KeySet *ks)
  * Every KeySet object that will be used must be initialized first, to setup
  * pointers, counters, etc. After use, all ksInit()ialized KeySets must be
  * cleaned with ksClear().
- * 
- * @deprecated Thus you can't get a Keyset without ksNew, this function is useless.
+ *
  * @see ksNew(), ksClose(), keyInit()
  * @return 1 on success
  */
-int ksInit(KeySet *ks) {
+int ksInit(KeySet *ks)
+{
 	ks->array = 0;
 
 	ks->size=0;
@@ -1921,10 +1960,11 @@ int ksInit(KeySet *ks) {
 }
 
 
-/*
+/**
+ * @internal
+ *
  * KeySet object initializer.
  *
- * @deprecated Thus you can't get a Keyset without ksNew, this function is useless.
  * @see ksDel(), ksNew(), keyInit()
  * @return 1 on success
  */
