@@ -28,8 +28,8 @@
 
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef HAVE_KDBCONFIG_H
+#include "kdbconfig.h"
 #endif
 
 #if DEBUG && HAVE_STDIO_H
@@ -52,25 +52,29 @@
 #include "kdbprivate.h"
 
 
-
-/**
- * Ask if key is marked for stat only.
+/**Clear flags of a key.
  *
- * Ask if the key will be stat instead of get it from the key database
- * completely doing kdbGetKey() or kdbGet(). This is useful
- * if you are not interested in the value, comment or key type.
+ * @todo Should be done only in kdbGet() part of plugins.
+ *
+ * If you want to get the current flags, just call
+ * it with semiflag set to 0.
  *
- * @see keyStat(), kdbGet()
  * @param key the key object to work with
- * @return 1 if it is marked, 0 otherwise
- * @return -1 on NULL pointer
+ * @return -1 on null key
+ * @return new flags for that key otherwise
  * @ingroup keytest
- **/
-int keyNeedStat(const Key *key)
+ *
+ */
+int keyClearSync (Key *key)
 {
 	if (!key) return -1;
 
-	return (key->flags & KEY_FLAG_STAT) == KEY_FLAG_STAT;
+	keyflag_t semiflag = KEY_FLAG_SYNC;
+
+	semiflag=~semiflag;
+	key->flags &= semiflag;
+
+	return key->flags;
 }
 
 
@@ -105,26 +109,6 @@ int keyNeedSync(const Key *key)
 }
 
 
-/**
- * Ask if key is marked for permanent remove.
- *
- * Ask if the key will be removed instead of writing in the key database
- * when doing kdbSetKey() or kdbSet().
- *
- * @see keyRemove()
- * @see kdbSet(), kdbSetKey(), kdbRemove()
- * @param key the key object to work with
- * @return 1 if it is marked, 0 otherwise
- * @return -1 on NULL pointer
- * @ingroup keytest*/
-int keyNeedRemove(const Key *key)
-{
-	if (!key) return -1;
-
-	return (key->flags & KEY_FLAG_REMOVE) == KEY_FLAG_REMOVE;
-}
-
-
 
 /**
  * Check whether a key is under the @p system namespace or not
@@ -165,24 +149,24 @@ int keyIsUser(const Key *key)
 	else return 0;
 }
 
-
-
 /**
  * Check if the key check is below the key key or not.
  *
- * @code
-Example:
+ * Example:
+@verbatim
 key user/sw/app
 check user/sw/app/key
-
-returns true because check is below key
-
-Example:
+@endverbatim
+ *
+ * returns true because check is below key
+ *
+ * Example:
+@verbatim
 key user/sw/app
 check user/sw/app/folder/key
-
-returns also true because check is indirect below key
- * @endcode
+@endverbatim
+ *
+ * returns also true because check is indirect below key
  *
  * @param key the key object to work with
  * @param check the key to find the relative position of
@@ -212,10 +196,29 @@ int keyIsBelow(const Key *key, const Key *check)
 	return 1;
 }
 
+/**
+ * Check if a key is below or same or not.
+ *
+ * @param key the key object to work with
+ * @see keyIsBelow()
+ */
+int keyIsBelowOrSame (const Key *key, const Key *check)
+{
+	if (!key || !check) return -1;
+
+	const char *name1 = keyName(key);
+	const char *name2 = keyName(check);
+
+	if (keyIsBelow (key, check)) return 1;
+	else if (!strcmp (name1, name2)) return 1;
+	return 0;
+}
+
+
 /**
  * Check if the key check is direct below the key key or not.
  *
- * @code
+@verbatim
 Example:
 key user/sw/app
 check user/sw/app/key
@@ -227,7 +230,7 @@ key user/sw/app
 check user/sw/app/folder/key
 
 does not return true, because there is only a indirect relation
- * @endcode
+@endverbatim
  *
  * @param key the key object to work with
  * @param check the key to find the relative position of
@@ -254,6 +257,149 @@ int keyIsDirectBelow(const Key *key, const Key *check)
 }
 
 
+/**
+ * Information about the relation in the hierarchy between
+ * two keys.
+ *
+ * Unlike keyCmp() the number gives information
+ * about hierarchical information.
+ *
+ *
+ * - If the keys are the same 0 is returned.
+ * So it is the key itself.
+@verbatim
+user/key
+user/key
+@endverbatim
+ *
+ *@code
+keySetName (key, "user/key/folder");
+keySetName (check, "user/key/folder");
+succeed_if (keyRel (key, check) == 0, "should be same");
+ *@endcode
+ *
+ * @note this relation can be checked with keyCmp() too.
+ *
+ *
+ * - If the key is direct below the other one 1 is returned.
+ * That means that, in terms of hierarchy, no other key is
+ * between them - it is a direct child.
+@verbatim
+user/key/folder
+user/key/folder/child
+@endverbatim
+ *
+ *@code
+keySetName (key, "user/key/folder");
+keySetName (check, "user/key/folder/child");
+succeed_if (keyRel (key, check) == 1, "should be direct below");
+ *@endcode
+ *
+ *
+ * - If the key is below the other one, but not directly 2 is returned.
+ * This is also called grand-child.
+@verbatim
+user/key/folder
+user/key/folder/any/depth/deeper/grand-child
+@endverbatim
+ *
+ *
+ *@code
+keySetName (key, "user/key/folder");
+keySetName (check, "user/key/folder/any/depth/deeper/grand-child");
+succeed_if (keyRel (key, check) >= 2, "should be below (but not direct)");
+succeed_if (keyRel (key, check) > 0, "should be below");
+succeed_if (keyRel (key, check) >= 0, "should be the same or below");
+ *@endcode
+ *
+ *
+ * - If a invalid or null ptr key is passed, -1 is returned
+ *
+ *
+ * - If the keys have no relations, but are not invalid, -2 is returned.
+ *
+ *
+ * - If the keys are in the same hierarchy, a value smaller then -2 is returned.
+ * It means that the key is not below.
+@verbatim
+user/key/myself
+user/key/sibling
+@endverbatim
+ *
+ * @code
+keySetName (key, "user/key/folder");
+keySetName (check, "user/notsame/folder");
+succeed_if (keyRel (key, check) < -2, "key is not below, but same namespace");
+ * @endcode
+ *
+ * @code
+ * @endcode
+ *
+ *
+ * TODO Below is an idea how it could be extended:
+ * It could continue the search into the other direction
+ * if any (grand-)parents are equal.
+ *
+ * - If the keys are direct below a key which is next to the key, -2 is returned.
+ * This is also called nephew. (TODO not implemented)
+ * @verbatim
+user/key/myself
+user/key/sibling
+@endverbatim
+ *
+ * - If the keys are direct below a key which is next to the key, -2 is returned.
+ * This is also called nephew. (TODO not implemented)
+ * @verbatim
+user/key/myself
+user/key/sibling/nephew
+@endverbatim
+ *
+ * - If the keys are below a key which is next to the key, -3 is returned.
+ * This is also called grand-nephew. (TODO not implemented)
+@verbatim
+user/key/myself
+user/key/sibling/any/depth/deeper/grand-nephew
+@endverbatim
+ *
+ * The same holds true for the other direction, but with negative values.
+ * For no relation INT_MIN is returned.
+ *
+ * @note to check if the keys are the same, you must use
+ *       keyCmp() == 0!
+ *       keyRel() does not give you the information if it did not
+ *       find a relation or if it is the same key.
+ *
+ * @return dependend on the relation:
+ *   2.. if below
+ *   1.. if direct below
+ *   0.. if the same
+ *  -1.. on null or invalid keys
+ *  -2.. if none of any other relation
+ *  -3.. if same hierarchy (none of those below)
+ *  -4.. if sibling (in same hierarchy)
+ *  -5.. if nephew (in same hierarchy)
+ *
+ * @param key the key object to work with
+ * @param check the second key object to check the relation with
+ * @ingroup keytest
+ */
+int keyRel (const Key *key, const Key *check)
+{
+	if (!key || !check) return -1;
+	if (!key->key || !check->key) return -1;
+
+	if (!keyCmp(key,check)) return 0;
+	if (keyIsDirectBelow(key, check)) return 1;
+	if (keyIsBelow(key, check)) return 2;
+	if (keyIsUser(key) && keyIsUser(check)) return -3;
+	if (keyIsSystem(key) && keyIsSystem(check)) return -3;
+	// if (keyIsSibling(key, check)) return -4;
+	// if (keyIsNephew(key, check)) return -5;
+
+	return -2;
+}
+
+
 /**
  * Check whether a key is inactive or not.
  *
@@ -295,11 +441,13 @@ int keyIsInactive (const Key *key)
 
 
 /**
- * Check if a key is directory key.
+ * Check if the mode for the key has access
+ * privileges.
  *
- * Folder keys may also have value and comment.
- * They are discern by having a executable bit
- * set.
+ * In the filesys backend a key represented through
+ * a file has the mode 664, but a key represented
+ * through a folder 775. keyIsDir() checks if all
+ * 3 executeable bits are set.
  *
  * If any executable bit is set it will be recognized
  * as a directory.
@@ -318,6 +466,9 @@ int keyIsInactive (const Key *key)
  * Accessing does not mean that you can get any value or
  * comments below, see @ref mode for more information.
  *
+ * @note currently mountpoints can only where keyIsDir()
+ *       is true (0.7.0) but this is likely to change.
+ *
  * @param key the key object to work with
  * @return 1 if key is a directory, 0 otherwise
  * @return -1 on NULL pointer
@@ -326,17 +477,22 @@ int keyIsInactive (const Key *key)
  */
 int keyIsDir(const Key *key)
 {
+	mode_t mode;
+
 	if (!key) return -1;
 
-	return ((key->mode & KEY_DEF_DIR) != 0);
+	mode = keyGetMode(key);
+
+	return ((mode & KEY_DEF_DIR) != 0);
 }
 
 
 /**
  * Check if a key is binary type.
  *
- * The function checks if the keytype is in the range between KEY_TYPE_BINARY and
- * less than excluding KEY_TYPE_STRING. Then it will be interpreted as binary.
+ * The function checks if the key is a binary. Opposed to string values binary
+ * values can have '\\0' inside the value and may not be terminated by a null
+ * character. Their disadvantage is that you need to pass their size.
  *
  * Make sure to use this function and don't test the binary type another way to
  * ensure compatibility and to write less error prone programs.
@@ -344,7 +500,6 @@ int keyIsDir(const Key *key)
  * @return 1 if it is binary
  * @return 0 if it is not
  * @return -1 on NULL pointer
- * @see keySetType() for more information on types
  * @see keyGetBinary(), keySetBinary()
  * @param key the key to check
  * @ingroup keytest
@@ -353,15 +508,15 @@ int keyIsBinary(const Key *key)
 {
 	if (!key) return -1;
 
-	return (KEY_TYPE_BINARY <= key->type && key->type < KEY_TYPE_STRING);
+	return keyGetMeta(key, "binary") != 0;
 }
 
 
 /**
  * Check if a key is string type.
  *
- * The function checks if the keytype is larger or equal KEY_TYPE_STRING.
- * Then it will be considered as string type.
+ * String values are null terminated and are not allowed to have any '\\0' characters
+ * inside the string.
  *
  * Make sure to use this function and don't test the string type another way to
  * ensure compatibility and to write less error prone programs.
@@ -369,7 +524,6 @@ int keyIsBinary(const Key *key)
  * @return 1 if it is string
  * @return 0 if it is not
  * @return -1 on NULL pointer
- * @see keySetType for more information on types
  * @see keyGetString(), keySetString()
  * @param key the key to check
  * @ingroup keytest
@@ -378,20 +532,19 @@ int keyIsString(const Key *key)
 {
 	if (!key) return -1;
 
-	return (key->type >= KEY_TYPE_STRING);
+	return keyGetMeta(key, "binary") == 0;
 }
 
 
 
 
 
-/*
+/**
  * Compare 2 keys.
  *
  * The returned flags bit array has 1s (differ) or 0s (equal) for each key
  * meta info compared, that can be logically ORed using @c #keyswitch_t flags.
- * The flags you can use are @link keyswitch_t::KEY_TYPE KEY_TYPE
- * @endlink, @link keyswitch_t::KEY_NAME KEY_NAME @endlink,
+ * @link keyswitch_t::KEY_NAME KEY_NAME @endlink,
  * @link keyswitch_t::KEY_VALUE KEY_VALUE @endlink,
  * @link keyswitch_t::KEY_OWNER KEY_OWNER @endlink,
  * @link keyswitch_t::KEY_COMMENT KEY_COMMENT @endlink,
@@ -473,7 +626,12 @@ kdbClose (handle);
  */
 keyswitch_t keyCompare(const Key *key1, const Key *key2)
 {
+	if (!key1 && !key2) return 0;
+	if (!key1 || !key2) return KEY_NULL;
+
 	keyswitch_t ret=0;
+	ssize_t nsize1 = keyGetNameSize(key1);
+	ssize_t nsize2 = keyGetNameSize(key2);
 	const char *name1 = keyName(key1);
 	const char *name2 = keyName(key2);
 	const char *comment1 = keyComment(key1);
@@ -482,17 +640,14 @@ keyswitch_t keyCompare(const Key *key1, const Key *key2)
 	const char *owner2 = keyOwner(key2);
 	const void *value1 = keyValue(key1);
 	const void *value2 = keyValue(key2);
-	int remove1 = keyNeedRemove(key1);
-	int remove2 = keyNeedRemove(key2);
 	ssize_t size1 = keyGetValueSize(key1);
 	ssize_t size2 = keyGetValueSize(key2);
 
 
-	if (key1->uid != key2->uid)        ret|=KEY_UID;
-	if (key1->gid != key2->gid)        ret|=KEY_GID;
-	if (key1->type != key2->type)      ret|=KEY_TYPE;
-	if ((key1->mode) != (key2->mode))  ret|=KEY_MODE;
-	if (remove1 != remove2)            ret|=KEY_REMOVE;
+	if (keyGetUID(key1) != keyGetUID(key2))  ret|=KEY_UID;
+	if (keyGetGID(key1) != keyGetGID(key2))  ret|=KEY_GID;
+	if (keyGetMode(key1)!= keyGetMode(key2)) ret|=KEY_MODE;
+	if (nsize1 != nsize2)              ret|=KEY_NAME;
 	if (strcmp(name1, name2))          ret|=KEY_NAME;
 	if (strcmp(comment1, comment2))    ret|=KEY_COMMENT;
 	if (strcmp(owner1, owner2))        ret|=KEY_OWNER;
